#region Copyright Syncfusion Inc. 2001 - 2008
//
//  Copyright Syncfusion Inc. 2001 - 2008. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.Text;
using System.Collections.Specialized;
using System.IO;
using System.Reflection;

using Syncfusion.ComponentModel;
using Syncfusion.Diagnostics;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms;
using Syncfusion.Collections.BinaryTree;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;


namespace DataGrid
{
    public class FilterButton : GridCellButton
    {
        public FilterButton(GridTextBoxCellRenderer control)
            : base(control)
        {
        }

        /// <override/>
        public override void Draw(Graphics g, int rowIndex, int colIndex, bool bActive, GridStyleInfo style)
        {
            TraceUtil.TraceCurrentMethodInfo(rowIndex, colIndex, bActive, style.CellValue);

            base.Draw(g, rowIndex, colIndex, bActive, style);

            // draw the button
            bool hovering = IsHovering(rowIndex, colIndex);
            bool mouseDown = IsMouseDown(rowIndex, colIndex);
            bool disabled = !style.Clickable;

            ButtonState buttonState = ButtonState.Normal;
            if (disabled)
                buttonState |= ButtonState.Inactive | ButtonState.Flat;

            else if (!hovering && !mouseDown)
                buttonState |= ButtonState.Flat;

            Point ptOffset = Point.Empty;
            if (mouseDown)
            {
                ptOffset = new Point(1, 1);
                buttonState |= ButtonState.Pushed;
            }

            GridTableFilterBarExtCellRenderer renderer = (GridTableFilterBarExtCellRenderer)Owner;
            //Get the bitmap images for compare operators
            Bitmap bm = renderer.Model.GetCompareOperatorImage(style);
            // with an existing bitmap. GridIconPaint is convenient because it lets
            // us easily draw over existing background and replace the black color
            // in the bitmap with a different color.
            //GridIconPaint.
            GridGroupingExtBitmaps.IconPainter.PaintIcon(g, Bounds, ptOffset, bm, Color.Black);

        }

    }

    public class ClearFilterButton : GridCellButton
    {

        public ClearFilterButton(GridTextBoxCellRenderer control)
            : base(control)
        {
        }

        /// <override/>
        public override void Draw(Graphics g, int rowIndex, int colIndex, bool bActive, GridStyleInfo style)
        {
            TraceUtil.TraceCurrentMethodInfo(rowIndex, colIndex, bActive, style.CellValue);

            base.Draw(g, rowIndex, colIndex, bActive, style);

            // draw the button
            bool hovering = IsHovering(rowIndex, colIndex);
            bool mouseDown = IsMouseDown(rowIndex, colIndex);
            bool disabled = !style.Clickable;

            ButtonState buttonState = ButtonState.Normal;
            if (disabled)
                buttonState |= ButtonState.Inactive | ButtonState.Flat;

            else if (!hovering && !mouseDown)
                buttonState |= ButtonState.Flat;

            Point ptOffset = Point.Empty;
            if (mouseDown)
            {
                ptOffset = new Point(1, 1);
                buttonState |= ButtonState.Pushed;
            }

            GridTableFilterBarExtCellRenderer renderer = (GridTableFilterBarExtCellRenderer)Owner;

            Bitmap bm = null;
            if (renderer.Model.HasFilter((style as GridTableCellStyleInfo).TableCellIdentity))
               bm = GridGroupingExtBitmaps.GetBitmap("filter_delete"); // make sure this is included in project and marked as "Embedded Resource"
            else
               bm = GridGroupingExtBitmaps.GetBitmap("filter");

            // Instead of using GridIconPaint you could simple use Image.Draw here
            // with an existing bitmap. GridIconPaint is convenient because it lets
            // us easily draw over existing background and replace the black color
            // in the bitmap with a different color.
            GridGroupingExtBitmaps.IconPainter.PaintIcon(g, Bounds, ptOffset, bm, Color.Blue);
        }
    }

}
