#region Copyright Syncfusion Inc. 2001 - 2008
//
//  Copyright Syncfusion Inc. 2001 - 2008. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Re-distribution in any form is strictly
//  prohibited. Any infringement will be prosecuted under applicable laws. 
//
#endregion

using System;
using System.Collections;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Globalization;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.Text;
using System.Collections.Specialized;
using System.IO;
using System.Reflection;
using Syncfusion.ComponentModel;
using Syncfusion.Diagnostics;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms;
using Syncfusion.Collections.BinaryTree;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Grid.Grouping;


namespace DataGrid
{
    #region GridTableFilterBarExtCell Model Class
    /// <summary>
	/// Implements the DataModel part for a ExtendedTableFilterBar cell.
	/// </summary>
	[Serializable]
    public class GridTableFilterBarExtCellModel : GridComboBoxCellModel
	{
		/// <overload>
		/// Initializes a new <see cref="GridTableFilterBarExtCellModel"/> object.
		/// </overload>
		/// <summary>
		/// Initializes a new <see cref="GridTableFilterBarExtCellModel"/> object 
		/// and stores a reference to the <see cref="GridModel"/> this cell belongs to.
		/// </summary>
		/// <param name="grid">The <see cref="GridModel"/> for this cell model.</param>
		/// <remarks>
		/// You typically access cell models through the <see cref="GridModel.CellModels"/>
		/// property of the <see cref="GridModel"/> class.
		/// </remarks>
        public GridTableFilterBarExtCellModel(GridModel grid)
			: base(grid)
		{
			AllowFloating = false;
			ButtonBarSize = new Size(ButtonWidth * 2, 0);
			SupportsChoiceList = true;
            AllowDoubleClickChangeSelectedIndex = false;
            
		}


        GridTableFilterBarExtCellRenderer renderer = null;
        GridTableFilterBarExtCellRenderer Renderer
        {
            get
            {
                return renderer;
            }
        }

		/// <summary>
		/// Initializes a new <see cref="GridTableFilterBarExtCellModel"/> from a serialization stream.
		/// </summary>
		/// <param name="info">An object that holds all the data needed to serialize or deserialize this instance.</param>
		/// <param name="context">Describes the source and destination of the serialized stream specified by info. </param>
		protected GridTableFilterBarExtCellModel(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			//TraceUtil.TraceCurrentMethodInfoIf(Switches.Serialization.TraceVerbose, info.FullTypeName, info.MemberCount);
            base.ButtonBarSize = new Size(ButtonWidth * 2, 0);
		}

        static GridTableFilterBarExtCellModel()
        {
            GridTableFilterBarExtCellModel.selectAllText = "(All)";
            GridTableFilterBarExtCellModel.selectCustomText = "(Custom...)";
            GridTableFilterBarExtCellModel.selectEmptyText = "(Empty)";
        }

        int buttonWidth = SystemInformation.VerticalScrollBarWidth;
        internal int ButtonWidth
        {
            get { return buttonWidth; }
            set { buttonWidth = value; }
        }


        GridGroupingControl GroupingGrid
        {
            get { return ((GridTableModel)this.Grid).GroupingControl; }
        }

        ListBox compareOprListBox = null;
        CustomFilters customFilters = null;
        Hashtable compareOprBitmaps;
        int originalCompareOperCount = 0;
        internal ListBox CompareOperatorListPart
        {
            get 
            {
                if (compareOprListBox == null)
                {
                    this.compareOprListBox = (ListBox)new GridComboBoxListBoxPart();
                    this.compareOprListBox.Tag = "CompareOperatorList";
                    //Add the compare operators in ListBox
                    this.compareOprListBox.Items.AddRange(new string[] { "Equals","Expression Match","StartsWith", "EndsWith" });
                    this.compareOprListBox.Items.Remove(Enum.GetName(typeof(FilterCompareOperator), FilterCompareOperator.Custom));
                    originalCompareOperCount = this.compareOprListBox.Items.Count;
                    if(customFilters != null)
                        customFilters = null;
                    customFilters = new CustomFilters();
                  
                    foreach (CustomFilters.ExpressionAndImage expImg in customFilters)
                        compareOprListBox.Items.Add(expImg.Name);
                    InitCompareOprBitmaps();
                }
                return compareOprListBox; 
            }
        }

        /// <summary>
        /// Initialize the Bitmap imagesforcompare operator
        /// </summary>
        void InitCompareOprBitmaps()
        {
            if (compareOprBitmaps != null)
                compareOprBitmaps = null;
            compareOprBitmaps = new Hashtable();

            
            StringCollection operatorNames = new StringCollection();
            operatorNames.AddRange(new string[] { "StartsWith", "EndsWith", "Equals", "NotEquals", "LessThan", "LessThanOrEqualTo",
            "GreaterThan", "GreaterThanOrEqualTo", "Like", "Expression Match"});

            for (int i = 0; i < operatorNames.Count; i++)
                compareOprBitmaps.Add(operatorNames[i], GridGroupingExtBitmaps.GetBitmap(operatorNames[i]));

            if (customFilters != null)
            {
                foreach (CustomFilters.ExpressionAndImage expImg in customFilters)
                {
                    if (expImg.Image != null)
                        compareOprBitmaps.Add(expImg.Name, expImg.Image);
                }
            }

        }

        Hashtable compareOperator = new Hashtable();

        internal void SetLogicalCompareOperator(object key, string name)
        {
            if (compareOperator.ContainsKey(key))
                compareOperator[key] = name;
            else
                compareOperator.Add(key, name);
        }
        /// <summary>
        /// return the selected compare operator name
        /// </summary>
        internal string GetCompareOperatorName(object key)
        {
            if (compareOperator.ContainsKey(key))
                return (string)compareOperator[key];
            else
                return string.Empty;
        }

        //return the bitmap images for compare operator
        internal Bitmap GetCompareOperatorImage(GridStyleInfo style)
        {
            GridTableCellStyleInfo tableCell = (GridTableCellStyleInfo)style;

            string name = GetCompareOperatorName(tableCell.TableCellIdentity.Column.Name);
            if (compareOprBitmaps == null)
                InitCompareOprBitmaps();

            if (Renderer.GetFilterBarText(style) == selectCustomText)
            {
                return GridGroupingExtBitmaps.GetBitmap("startswith");
            }

            if (name == string.Empty)
            {
                return GridGroupingExtBitmaps.GetBitmap("equals");
            }

            if (compareOprBitmaps.ContainsKey(name))
            {
                if (name == "Equals")
                    return GridGroupingExtBitmaps.GetBitmap("equals");
                else if (name == "EndsWith")
                    return GridGroupingExtBitmaps.GetBitmap("endswith");
                else if (name == "Expression Match")
                    return GridGroupingExtBitmaps.GetBitmap("match");
                else if (name == "StartsWith")
                    return GridGroupingExtBitmaps.GetBitmap("startswith");
            }

            if (customFilters != null && customFilters.Contains(name))
            {
                Bitmap bm = customFilters.GetImage(name);
                if (bm != null)
                    return bm;
            }

            //Default Filter Bitmap
            return GridGroupingExtBitmaps.GetBitmap("default");
        }

        /// <summary>
        /// return the compare operator name for select the item 
        /// </summary>
        protected object GetLogicalCompareOperator(string key)
        {
            string name = GetCompareOperatorName(key);
            switch (name)
            {
                case "Equals":
                    return FilterCompareOperator.Equals;
                case "Like":
                case "StartsWith":
                case "EndsWith":
                    return FilterCompareOperator.Like;
                case "Expression Match":
                    return FilterCompareOperator.Match;
                default:
                    return FilterCompareOperator.Custom;
            }
        }


        RecordFilterDescriptorCollection recordFilters;
        TableDescriptor tableDescriptor;
        Syncfusion.Grouping.Table table;


        public void ApplyFilters()
        {
            if (tableDescriptor != null && recordFilters != null)
            {
                tableDescriptor.RecordFilters.InitializeFrom(recordFilters);
            }
            recordFilters = null;
        }

        /// <summary>
        /// Fill the choices for listbox
        /// </summary>
        public override void FillWithChoices(ListBox listBox, GridStyleInfo style, out bool exclusive)
        {
            exclusive = false;
            if (listBox.Tag == null || listBox.Tag.ToString() != "CompareOperatorList")
            {
                GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo)style;

                object[] items = (object[])GetFilterBarChoices(tableStyleInfo.TableCellIdentity);

                listBox.Items.Clear();

                if (items != null)
                {
                    listBox.Items.Add(SelectAllText);

                    foreach (object item in items)
                    {
                        if (item is DBNull)
                            listBox.Items.Add(SelectEmptyText);

                        else if (item != null)
                            listBox.Items.Add(style.GetFormattedText(item));
                    }
                }
            }
        }

        /// <summary>
        /// Returns the unique choices to be displayed in the filterbar cell.
        /// </summary>
        /// <param name="tableCellIdentity"></param>
        /// <returns></returns>
        public virtual object[] GetFilterBarChoices(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            if (tableCellIdentity.FilterBarSummaryDescriptor != null)
            {
                ITreeTableSummary[] summaries = tableCellIdentity.DisplayElement.ParentGroup.GetSummaries(tableCellIdentity.Table);
                int summaryIndex = tableCellIdentity.Table.TableDescriptor.Summaries.IndexOf(tableCellIdentity.FilterBarSummaryDescriptor);
                FilterBarChoicesSummary filterBarChoicesSummary = (FilterBarChoicesSummary)summaries[summaryIndex];
                return filterBarChoicesSummary.Values;
            }

            object[] result = null;
            GridGroupingControl gc = tableCellIdentity.Table.Engine.ParentControl;
            GridQueryFilterBarChoicesEventArgs qe = new GridQueryFilterBarChoicesEventArgs(tableCellIdentity.Column, false, tableCellIdentity.DisplayElement);
            gc.OnQueryFilterBarChoices(qe);
            if (!qe.Cancel)
                result = qe.UniqueFilterBarValues;


            return result;
        }
        /// <summary>
        /// Filter the record based on the filter name
        /// </summary>
        public RecordFilterDescriptor GetRecordFilter(RecordFilterDescriptorCollection recordFilters, GridTableCellStyleInfoIdentity tableCellIdentity, string filterName)
        {
            RecordFilterDescriptor filter = null;

            GridTableDescriptor td = tableCellIdentity.Table.TableDescriptor;
            GridRelationDescriptor rd = td.ParentRelation;
            if (rd == null || rd.RelationKind != RelationKind.UniformChildList)
            {
                filter = recordFilters[filterName];
            }
            else
            {
                object[] uniqueId = tableCellIdentity.DisplayElement.ParentGroup.UniqueGroupId;
                string mappingName = tableCellIdentity.Column.MappingName;
                foreach (RecordFilterDescriptor rfd in recordFilters)
                {
                    if (rfd.MappingName == mappingName && rfd.CompareUniqueId(uniqueId))
                    {
                        filter = rfd;
                        break;
                    }
                }
            }
            return filter;
        }

        public string GetUniqueColumnGroupId(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            Group g = tableCellIdentity.DisplayElement.ParentGroup;
            object[] id = g.UniqueGroupId;

            if (id == null || id.Length == 0)
                return tableCellIdentity.Column.Name;
            else
                return tableCellIdentity.Column.Name + "@" + g.UniqueGroupIdsToString();//.UniqueGroupId;
        }
        public object[] GetUniqueGroupId(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            Group g = tableCellIdentity.DisplayElement.ParentGroup;
            return g.UniqueGroupId;
        }

        public bool HasFilter(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            return (tableCellIdentity.Table.TableDescriptor.RecordFilters.Count > 0);
        }

        internal void ResetFilterBar(GridTableCellStyleInfoIdentity tableCellIdentity)
        {
            string filterName = GetUniqueColumnGroupId(tableCellIdentity);

            tableDescriptor = tableCellIdentity.Table.TableDescriptor;
            table = tableCellIdentity.Table;
            recordFilters = new RecordFilterDescriptorCollection();
            recordFilters.InitializeFrom(tableCellIdentity.Table.TableDescriptor.RecordFilters);

            RecordFilterDescriptor filter = recordFilters[filterName];
            if (filter != null)
            {
                recordFilters.Remove(filterName);
            }
            else
            {
                recordFilters.Clear();
            }
        }

        /// <summary>
        /// Applies a filter criteria. Note: The first two entries are reserved for (All) and (Custom).
        /// An index greater than one represents a valid choice found with GetFilterBarChoices.
        /// </summary>
        /// <param name="tableCellIdentity"></param>
        /// <param name="index"></param>
        public void Select(GridTableCellStyleInfoIdentity tableCellIdentity, int index)
        {
            Select(tableCellIdentity, null, index);
        }

        internal void Select(GridTableCellStyleInfoIdentity tableCellIdentity, object value, int index)
        {
            if (index >= 0)
            {
                if (index == 0)
                {
                    ResetFilterBar(tableCellIdentity);
                }
                else
                {
                    SelectItem(tableCellIdentity, index - 1);
                }
            }
            else if (index == -101 && value != null)
            {
                //Entered text filter
                if (value.ToString().Length > 0)
                    SelectItem(tableCellIdentity, GetLogicalCompareOperator(tableCellIdentity.Column.Name), IncludeFilterFormat(value, tableCellIdentity.Column.Name), -101);
                else
                    ResetFilterBar(tableCellIdentity);
            }

        }
        /// <summary>
        /// Filter the value based on typing value
        /// </summary>
        internal string IncludeFilterFormat(object value, object key)
        {
            string oprName = GetCompareOperatorName(key);
            string filter = Convert.ToString(value);
            switch (oprName)
            {
                case "StartsWith":
                    filter = string.Format("{0}*", value);
                    break;
                case "EndsWith":
                    filter = string.Format("*{0}", value);
                    break;
                case "Custom":
                    filter = string.Format("{0}", value);
                    break;
                default:
                    if (customFilters != null && customFilters.Contains(oprName))
                    {
                        string format = customFilters.GetExpression(oprName);
                        format = format.Replace("{VALUE}", "{0}");
                        filter = string.Format(format, value);
                        filter = string.Format("[{0}] {1}", key, filter);
                    }
                    break;
            }

            return filter;
        }

        internal bool IsExpressionFilter(GridTableCellStyleInfo style)
        {
            string name = GetCompareOperatorName(style.TableCellIdentity.Column.Name);
            if (customFilters != null && customFilters.Contains(name))
                return true;
            else
                return false;
        }
        /// <summary>
        /// Filter the value based on typed value
        /// </summary>
        internal virtual string ExcludeFilterFormat(object value, object key)
        {
            string oprName = GetCompareOperatorName(key);
            string filter = Convert.ToString(value);
            switch (oprName)
            {
                case "StartsWith":
                    if (filter.EndsWith("*"))
                        filter = filter.Remove(filter.Length - 1);
                    break;
                case "EndsWith":
                    if (filter.StartsWith("*"))
                        filter = filter.Remove(0, 1);
                    break;

                default:
                    if (this.customFilters != null && this.customFilters.Contains(oprName))
                    {
                        string format = this.customFilters.GetExpression(oprName);
                        format = format.Replace("{VALUE}", "{0}");
                        filter = string.Format(format, value);
                        filter = string.Format("[{0}] {1}", key, filter);
                    }

                    break;
            }

            return filter;
        }


        internal static string selectAllText;
        internal static string selectCustomText;
        internal static string selectEmptyText;

        /// <summary>
        /// Default: (All) 
        /// </summary>
        public string SelectAllText
        {
            get
            {
                return selectAllText;
            }
            set
            {
                selectAllText = value;
            }
        }

        /// <summary>
        /// Default: (Custom...)
        /// </summary>
        public string SelectCustomText
        {
            get
            {
                return selectCustomText;
            }
            set
            {
                selectCustomText = value;
            }
        }

        /// <summary>
        /// Default: (Empty)
        /// </summary>
        public string SelectEmptyText
        {
            get
            {
                return selectEmptyText;
            }
            set
            {
                selectEmptyText = value;
            }
        }

        protected void SelectItem(GridTableCellStyleInfoIdentity tableCellIdentity, int index)
        {
            SelectItem(tableCellIdentity, GetLogicalCompareOperator(tableCellIdentity.Column.Name), null, index);
        }

        /// <summary>
        /// Add the condions and value in RecordFieldDescriptor for Apply the filter
        /// </summary>
        protected void SelectItem(GridTableCellStyleInfoIdentity tableCellIdentity, object condition, object value, int index)
        {
            string filterName = GetUniqueColumnGroupId(tableCellIdentity);

            tableDescriptor = tableCellIdentity.Table.TableDescriptor;
            table = tableCellIdentity.Table;
            recordFilters = new RecordFilterDescriptorCollection();
            recordFilters.InitializeFrom(tableCellIdentity.Table.TableDescriptor.RecordFilters);

            RecordFilterDescriptor filter = GetRecordFilter(recordFilters, tableCellIdentity, filterName);

            object[] items = GetFilterBarChoices(tableCellIdentity);
            value = (index == -101) ? value : items[index];
            RecordFilterDescriptor newFilter = new RecordFilterDescriptor();
            newFilter.Name = filterName;
            newFilter.MappingName = tableCellIdentity.Column.MappingName;
            newFilter.UniqueGroupId = GetUniqueGroupId(tableCellIdentity);
            if (condition is FilterCompareOperator)
                newFilter.Conditions.Add(new FilterCondition((FilterCompareOperator)condition, value));
            else
                newFilter.Expression = (string)value;
                
    
            if (filter == null)
                recordFilters.Add(newFilter);
            else
                filter.InitializeFrom(newFilter);
      
        }
        /// <summary>
		/// Creates a <see cref="GridTableFilterBarExtCellRenderer"/> for this cell model that is specific to the GridControlBase.
		/// </summary>
		/// <param name="control">The <see cref="GridControlBase"/> the cell renderer is created for.</param>
		/// <returns>A new <see cref="GridTableFilterBarCellRenderer"/> specific for a <see cref="GridControlBase"/>.</returns>
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
            renderer = new GridTableFilterBarExtCellRenderer(control, this);
            return renderer;
		}

	}
    
    #endregion

    #region GridTableFilterBarExtCell Renderer Class
    /// <summary>
	/// Implements the renderer part of a TableFilterBar cell.
	/// </summary>
    public class GridTableFilterBarExtCellRenderer : GridComboBoxCellRenderer
    {
        
        GridTableCellStyleInfoIdentity tableCellIdentity;
        ListBox listBoxPart = null;

        /// <summary>
        /// Initializes a new <see cref="GridTableFilterBarCellRenderer"/> object for the given GridControlBase
        /// and <see cref="GridCellModelBase"/>.
        /// </summary>
        /// <param name="grid">The <see cref="GridControlBase"/> that displays this cell renderer.</param>
        /// <param name="cellModel">The <see cref="GridTableFilterBarCellExtModel"/> that holds data for this cell renderer that should
        /// be shared among views.</param>
        /// <remarks>References to GridControlBase 
        /// and GridCellModelBase will be saved.</remarks>
        public GridTableFilterBarExtCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
            : base(grid, cellModel)
        {
            AddButton(new FilterButton(this));
            AddButton(new ClearFilterButton(this));
        }
        


        #region ListBoxes [Unique Column Value Choise List/ CompareOperator Choise List]

        internal ListBox InternalListBoxPart
        {
            get { return this.listBoxPart; }
        }


        bool isComparerListBox = false;
        bool IsComparerListBoxPart
        {
            get { return isComparerListBox; }
            set
            {
                if (isComparerListBox != value)
                {
                    isComparerListBox = value;
                    if (isComparerListBox)
                        AttachComparerListBoxPart();
                    else
                        AttachOriginalListBoxPart();
                }
                
                if (!isComparerListBox)
                {
                    if (ListBoxVersion < 0)
                    {
                        GridTableCellStyleInfo style = Grid.GetTableViewStyleInfo(RowIndex, ColIndex);
                        if (style.IsTableCell)
                        {
                            bool exclusive;
                            Model.FillWithChoices(this.ListBoxPart, style, out exclusive);
                        }
                    }
                    EnsureOriginalListBoxPart();
                }
            }
        }

        string oldColumn = string.Empty;
        string currColumn = string.Empty;
        int ListBoxVersion
        {
            [DebuggerStepThrough()]
            get
            {
                if (currColumn.Equals(string.Empty) || !currColumn.Equals(oldColumn))
                {
                    oldColumn = currColumn;
                    return -1;
                }
                else
                    return 1;
            }
        }

        /// <summary>
        /// Called to attach a list box part to this renderer object.
        /// </summary>
        protected void AttachComparerListBoxPart()
        {
            //listBoxPart = (ListBox)this.DropDownContainer.Controls[0];
            EnsureOriginalListBoxPart();
            if(this.DropDownContainer.Controls.Contains(InternalListBoxPart))
                this.DropDownContainer.Controls.Remove(InternalListBoxPart);
            this.ListBoxPart = Model.CompareOperatorListPart;
            this.DropDownContainer.Controls.Add(Model.CompareOperatorListPart);
        }

        /// <summary>
        /// Called to attach a list box part to this renderer object.
        /// </summary>
        protected void AttachOriginalListBoxPart()
        {
            if (!this.DropDownContainer.Controls.Contains(InternalListBoxPart))
            {
                this.DropDownContainer.Controls.RemoveAt(0);
                this.ListBoxPart = InternalListBoxPart;
                this.DropDownContainer.Controls.Add(InternalListBoxPart);
            }
        }

        private void EnsureOriginalListBoxPart()
        {
            ListBox lb = (ListBox)this.DropDownContainer.Controls[0];
            if (lb.Tag == null || lb.Tag.ToString() != "CompareOperatorList")
                    listBoxPart = lb;
        }

       #endregion

        #region Buttons Related
        protected override void WireModel(GridCellModelBase cellModel)
        {
            Grid.CurrentCellMoved += new GridCurrentCellMovedEventHandler(Grid_CurrentCellMoved);
            base.WireModel(cellModel);
            
        }

        protected override void UnwireModel(GridCellModelBase cellModel)
        {
            Grid.CurrentCellMoved -= new GridCurrentCellMovedEventHandler(Grid_CurrentCellMoved);
            base.UnwireModel(cellModel);
        }

        void Grid_CurrentCellMoved(object sender, GridCurrentCellMovedEventArgs e)
        {
            if (Grid.CurrentCell != null)
                Grid.CurrentCell.Refresh();
              
            //version--;
        }

        protected override void OnButtonClicked(int rowIndex, int colIndex, int button)
        {
            //Clear Filter
            if (button == 2)
            {
                GridTableCellStyleInfo style = Grid.GetTableViewStyleInfo(rowIndex, colIndex);
                Model.ResetFilterBar(style.TableCellIdentity);
                Model.ApplyFilters();
            }
            else
            {
                currColumn = GetKey(rowIndex, colIndex).ToString();
                
                if (button == 0)
                {
                    IsComparerListBoxPart = false;
                }
                //Set Filter criteria
                else if (button == 1)
                {

                    IsComparerListBoxPart = true;

                }
                base.OnButtonClicked(rowIndex, colIndex, button);
            }
        }


        protected override Rectangle OnLayout(int rowIndex, int colIndex, GridStyleInfo style, Rectangle innerBounds, Rectangle[] buttonsBounds)
        {
            TraceUtil.TraceCurrentMethodInfo(rowIndex, colIndex, style, innerBounds, buttonsBounds);

            Rectangle buttonArea = Rectangle.FromLTRB(innerBounds.Left, innerBounds.Top, innerBounds.Left + Model.ButtonWidth, innerBounds.Bottom);
            buttonsBounds[2] = GridUtil.CenterInRect(buttonArea, new Size(Model.ButtonWidth, 20));

            buttonArea = Rectangle.FromLTRB(innerBounds.Right - Model.ButtonWidth, innerBounds.Top, innerBounds.Right, innerBounds.Bottom);
            buttonsBounds[1] = GridUtil.CenterInRect(buttonArea, new Size(Model.ButtonWidth, 20));

            buttonArea = Rectangle.FromLTRB(innerBounds.Right - Model.ButtonWidth * 2, innerBounds.Top, innerBounds.Right - Model.ButtonWidth, innerBounds.Bottom);
            buttonsBounds[0] = GridUtil.CenterInRect(buttonArea, new Size(Model.ButtonWidth, 20));

            innerBounds.Width -= Model.ButtonWidth * 2;

            //if (CanShowFilterButtons(rowIndex, colIndex))
                return new Rectangle(innerBounds.X + Model.ButtonWidth, innerBounds.Y, innerBounds.Width - Model.ButtonWidth, innerBounds.Height);
            
            //return innerBounds;
        }


        protected override void OnDrawCellButton(GridCellButton button, Graphics g, int rowIndex, int colIndex, bool bActive, GridStyleInfo style)
        {
            if (!(button is ClearFilterButton) || CanShowFilterButtons(rowIndex, colIndex))
                base.OnDrawCellButton(button, g, rowIndex, colIndex, bActive, style);
        }

        bool CanShowFilterButtons(int rowIndex, int colIndex)
        {
            if (Grid.CurrentCell.HasCurrentCellAt(rowIndex, colIndex))
                return true;

            bool mouseHover = false;
            GridRangeInfo key = GridRangeInfo.Cell(rowIndex, colIndex);
            if (mouseHoverAtCells.ContainsKey(key))
                mouseHover = (bool)mouseHoverAtCells[key];

                if (!mouseHover)
                    return false;
                
            return true;
        }

        protected override void OnMouseHoverEnter(int rowIndex, int colIndex)
        {
            GridRangeInfo key = GridRangeInfo.Cell(rowIndex, colIndex);
            if (mouseHoverAtCells.ContainsKey(key))
            {
                mouseHoverAtCells[key] = true;
            }
            else
                mouseHoverAtCells.Add(key, true);

            base.OnMouseHoverEnter(rowIndex, colIndex);

            Grid.RefreshRange(key);
        }


        protected override void OnMouseHoverLeave(int rowIndex, int colIndex, EventArgs e)
        {
            GridRangeInfo key = GridRangeInfo.Cell(rowIndex, colIndex);
            if (mouseHoverAtCells.ContainsKey(key))
            {
                mouseHoverAtCells[key] = false;
            }
            else
                mouseHoverAtCells.Add(key, false);

            base.OnMouseHoverLeave(rowIndex, colIndex, e);
            
            Grid.RefreshRange(key);
        }

       
        Hashtable mouseHoverAtCells = new Hashtable();
        #endregion

        #region Key Overrides
        protected override void OnKeyPress(KeyPressEventArgs e)
        {
            ListBox listBoxPart = this.ListBoxPart;
            GridComboBoxCellModel model = Model as GridComboBoxCellModel;

            if (!e.Handled)
            {
                GridTableCellStyleInfo style = Grid.GetTableViewStyleInfo(RowIndex, ColIndex);
                if (!this.HasFocusControl)
                {
                    // Key pressed for the first time.
                    if (!Char.IsControl(e.KeyChar) && !IsReadOnly())
                    {
                        if (listBoxPart.Items == null)
                        {
                            object ds = model.GetDataSource(this.StyleInfo);
                            if (ds == null)
                            {
                                base.OnKeyPress(e);
                                return;
                            }
                            listBoxPart.DataSource = ds;
                        }

                        // Combo box mode with editing.
                        CurrentCell.BeginEdit();
                        SetTextBoxText(e.KeyChar.ToString(), true);
                        Model.Select(style.TableCellIdentity, e.KeyChar.ToString(), -101);
                        ApplyFilter();

                        if (this.IsEditing)
                            this.TextBoxControl.Select(1, Math.Max(0, this.TextBoxControl.Text.Length - 1));


                        e.Handled = true;
                    }
                }
                else
                {
                    // Key pressed after cell has been switched into edit mode.
                    char charCode = (char)(e.KeyChar & 255);

                    // Pass the e.KeyChar to the IsControl method, not the charCode.
                    if ((!Char.IsControl(e.KeyChar) || charCode == 8) && !IsReadOnly())
                    {
                        if (listBoxPart.Items == null)
                        {
                            object ds = model.GetDataSource(this.StyleInfo);
                            if (ds == null)
                            {
                                base.OnKeyPress(e);
                                return;
                            }

                            listBoxPart.DataSource = ds;
                        }

                        string s = TextBoxText + (!Char.IsControl(e.KeyChar) ? e.KeyChar.ToString() : "");
                        s = s.ToUpper();
                        if (this.ValidateString(s))
                        {
                            if (this.NotifyCurrentCellChanging())
                            {
                                CurrentCell.IsModified = true;
                                SetTextBoxText(s, true);

                                Model.Select(style.TableCellIdentity, s, -101);
                                ApplyFilter();
                                //TextBoxText = s;
                            }
                        }
                        TextBox.SelectionStart = TextBox.Text.Length;
                        e.Handled = true;
                    }
                }
            }
        }

        protected override void OnKeyDown(KeyEventArgs e)
        {
            object savedEditState = this.GetEditState();
            bool ignoreWmChar = true;

            if (!e.Handled)
            {

                switch (e.KeyCode)
                {

                    case Keys.Back:
                    case Keys.Delete:
                        if (!CurrentCell.HasControlFocus || !CurrentCell.IsEditing)
                        {
                            if (Control.ModifierKeys != Keys.Alt)
                            {
                                if (Grid.ShouldDeleteKeyClearCurrentCellContentsOnly())
                                {
                                    if (!IsReadOnly()        // cell is not readonly
                                        && OnDeleting()
                                        && Grid.RaiseCurrentCellDeleting()       // OnDeleteCell notification returns true
                                        )
                                    {
                                        // Delete text, SetTextBoxText call BeginEdit()
                                        SetTextBoxText("", true);
                                        CurrentCell.IsModified = true;
                                        CurrentCell.ScrollInView(GridScrollCurrentCellReason.KeyPress);
                                    }
                                    e.Handled = true;
                                }
                            }
                        }
                        else if (Grid.IsWindowless || TextBox is TextBox)  // TextBox is "Original TextBox", not RichTextbox ...
                        {
                            if (TextBox.SelectionLength > 0)
                                TextBox.SelectedText = "";
                            else if (TextBox.SelectionStart >= 0)
                            {
                                if (e.KeyCode == Keys.Back)
                                {
                                    if (TextBox.SelectionStart > 0)
                                        TextBox.Select(TextBox.SelectionStart - 1, 1);
                                }
                                else
                                    TextBox.Select(TextBox.SelectionStart, 1);
                                TextBox.SelectedText = "";
                            }
                            e.Handled = true;
                            ignoreWmChar = TextBox is GridOriginalTextBoxControl && TextBox.SelectionLength == 0;
                        }
                        break;

                    case Keys.Escape:
                        if (CurrentCell.IsDroppedDown)
                        {
                            CurrentCell.CloseDropDown(PopupCloseType.Canceled);
                            e.Handled = true;
                        }
                        else if (CurrentCell.IsModified)
                        {
                            CurrentCell.RejectChanges();
                            CurrentCell.CancelEdit();
                            CurrentCell.Refresh();
                            CurrentCell.ScrollInView(GridScrollCurrentCellReason.KeyPress);
                            e.Handled = true;
                        }
                        else
                            CurrentCell.CancelEdit();

                        if ((Grid.Model.Options.ActivateCurrentCellBehavior & Syncfusion.Windows.Forms.Grid.GridCellActivateAction.SetCurrent) != 0)
                            CurrentCell.BeginEdit();
                        return;

                    case Keys.End:
                        if (Control.ModifierKeys == Keys.None && !CurrentCell.HasControlFocus && !this.DisableTextBox)
                        {

                            if (CurrentCell.BeginEdit())
                            {
                                // Position caret.
                                CurrentCell.HasControlFocus = true;
                                TextBox.SelectionStart = TextBox.Text.Length;
                                TextBox.SelectionLength = 0;
                                CurrentCell.ScrollInView(GridScrollCurrentCellReason.KeyPress);
                                e.Handled = true;
                            }
                        }
                        break;

                    case Keys.Home:
                        if (Control.ModifierKeys == Keys.None && !CurrentCell.HasControlFocus && !this.DisableTextBox)
                        {
                            if (CurrentCell.BeginEdit())
                            {
                                // Position caret.
                                CurrentCell.HasControlFocus = true;
                                TextBox.SelectionStart = 0;
                                TextBox.SelectionLength = 0;
                                CurrentCell.ScrollInView(GridScrollCurrentCellReason.KeyPress);
                                Grid.Update();
                                e.Handled = true;
                            }
                        }
                        break;

                    case Keys.Enter:
                        if (Control.ModifierKeys == Keys.None && CurrentCell.IsEditing && this.StyleInfo.AllowEnter && this.NotifyCurrentCellChanging())
                        {
                            SetSelectedText("\n", true);
                            CurrentCell.ScrollInView(GridScrollCurrentCellReason.KeyPress);
                            e.Handled = true;
                        }
                        break;

                    case Keys.Tab:
                        if (Grid.WantTabKey)
                            ignoreWmChar = true;
                        break;

                    case Keys.NumPad0:
                    case Keys.NumPad1:
                    case Keys.NumPad2:
                    case Keys.NumPad3:
                    case Keys.NumPad4:
                    case Keys.NumPad5:
                    case Keys.NumPad6:
                    case Keys.NumPad7:
                    case Keys.NumPad8:
                    case Keys.NumPad9:
                    case Keys.Left:
                    case Keys.Right:
                    case Keys.Up:
                    case Keys.Down:
                        base.OnKeyDown(e);
                        break;
                    default:
                        e.Handled = true;
                        break;
                }

            }
        }
        #endregion

        #region Filter Related
        void ApplyFilter()
        {
            Grid.CurrentCell.Lock();
            Model.ApplyFilters();
            Grid.CurrentCell.Unlock();
        }

        object GetKey()
        {
            return GetKey(RowIndex, ColIndex);
        }

        object GetKey(int rowIndex, int colIndex)
        {
            GridTableCellStyleInfo style = Grid.GetTableViewStyleInfo(colIndex, colIndex);
            if (!style.IsTableCell || style.TableCellIdentity.Column == null)
                return string.Empty;

            return style.TableCellIdentity.Column.Name;
        }

        public override string ControlText
        {
            get
            {
                return base.ControlText;
            }
            //set
            //{
            //    SetTextBoxText(value/*GetFilterBarText(StyleInfo)*/, true);// don't call base class - ignore.
            //}
        }

        /// <summary>
        /// Determines the text from record filter criteria that should be displayed in filterbar cell.
        /// </summary>
        /// <param name="style"></param>
        /// <returns></returns>
        public string GetFilterBarText(GridStyleInfo style)
        {
            GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo)style;
            GridTableCellStyleInfoIdentity tableCellIdentity = tableStyleInfo.TableCellIdentity;
            GridTableDescriptor td = tableCellIdentity.Table.TableDescriptor;
            string filterName = Model.GetUniqueColumnGroupId(tableCellIdentity);
            RecordFilterDescriptor filter = Model.GetRecordFilter(td.RecordFilters, tableCellIdentity, filterName);
            object value = Model.SelectCustomText;
            if (filter == null)
                value = "";
            else if (filter.Conditions.Count == 1)
            {
                if (filter.Conditions[0].CompareOperator == FilterCompareOperator.Equals)
                    value = filter.Conditions[0].CompareValue;
                else
                {
                    value = filter.Conditions[0].CompareValue;
                }
            }
            else if (Model.IsExpressionFilter(tableStyleInfo))
                value = filter.Expression;
            
            return Model.ExcludeFilterFormat(value, tableStyleInfo.TableCellIdentity.Column.Name);
        }

        public override void OnPrepareViewStyleInfo(GridPrepareViewStyleInfoEventArgs e)
        {
            e.Style.CellValue = GetFilterBarText(e.Style);

            e.Style.DropDownStyle = GridDropDownStyle.Editable;
            e.Style.ExclusiveChoiceList = false;
            //e.Style.ShowButtons = GridShowButtons.Show;
            e.Style.Clickable = true;

            base.OnPrepareViewStyleInfo(e);

        }

        protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
        {
            style.CellValue = GetFilterBarText(style);

            base.OnDraw(g, clientRectangle, rowIndex, colIndex, style);

            Point ptOffset = Point.Empty;
        }

        #endregion
        
        #region DropDown Overrides

        /// <summary>
        /// Handles the <see cref="Control.MouseUp"/> event of the list box.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">A <see cref="MouseEventArgs"/> with event data.</param>
        protected override void ListBoxMouseUp(object sender, MouseEventArgs e)
        {
            CurrentCell.CloseDropDown(PopupCloseType.Done);
            if (!IsComparerListBoxPart)
            {
                Model.Select(tableCellIdentity, this.ListBoxPart.SelectedIndex);

                // Reset current element before applying filter.
                Grid.GroupingControl.Table.CurrentRecordManager.NavigateTo(null);

                // Apply filter.
                Model.ApplyFilters();
            }
            else
            {
                GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo)StyleInfo;
                string value = GetFilterBarText(StyleInfo);
                Model.SetLogicalCompareOperator(tableStyleInfo.TableCellIdentity.Column.Name, Convert.ToString(ListBoxPart.SelectedItem));
                if (value.Length > 0)
                {
                    Grid.GroupingControl.Table.CurrentRecordManager.NavigateTo(null);
                    Model.Select(tableStyleInfo.TableCellIdentity, value, -101);
                    ApplyFilter();
                }
                Grid.CurrentCell.Refresh();

            }
        }

        #endregion

        /// <summary>
        /// A reference to the cell model.
        /// </summary>
        public new GridTableFilterBarExtCellModel Model
        {
            get
            {
                return (GridTableFilterBarExtCellModel)base.Model;
            }
        }

        /// <summary>
        /// A reference to the parent grid.
        /// </summary>
        public new GridTableControl Grid
        {
            get
            {
                return (GridTableControl)base.Grid;
            }
        }
        
        protected override void OnInitialize(int rowIndex, int colIndex)
        {

            GridTableCellStyleInfo tableStyleInfo = (GridTableCellStyleInfo)StyleInfo;
            tableCellIdentity = tableStyleInfo.TableCellIdentity;
            
            Grid.Table.CurrentRecord = null;
            
            base.OnInitialize(rowIndex, colIndex);
            
        }
    }
    #endregion

    #region Filter Table Class
    public class CustomFilters : IEnumerator, IEnumerable
    {
        public CustomFilters()
        {
            expressionFilters = new Hashtable();
        }

        Hashtable expressionFilters;

        public int Count
        {
            get { return expressionFilters.Count; }
        }

        public bool Contains(string name)
        {
            return expressionFilters.ContainsKey(name);
        }

        internal ExpressionAndImage this[string key]
        {
            get
            {
                return (ExpressionAndImage)expressionFilters[key];
            }
        }

        public string GetExpression(string key)
        {
            ExpressionAndImage expImg = (ExpressionAndImage)expressionFilters[key];
            return expImg.Expression;
        }

        public Bitmap GetImage(string key)
        {
            ExpressionAndImage expImg = (ExpressionAndImage)expressionFilters[key];
            return expImg.Image;
        }

        /// <summary>
        /// Add a filter expression to the CompareOperaatorList.
        /// </summary>
        /// <param name="name">Name of the filter expression.</param>
        /// <param name="expression">A formula expression with {VALUE} token.</param>
        public void Add(string name, string expression)
        {
            ExpressionAndImage expImg = new ExpressionAndImage(name);
            expImg.Expression = expression;
            expressionFilters.Add(name, expImg);
        }

        /// <summary>
        /// Add a filter expression to the CompareOperaatorList.
        /// </summary>
        /// <param name="name">Name of the filter expression.</param>
        /// <param name="expression">A formula expression with {VALUE} token.</param>
        /// <param name="img">Bitmap to display in the button for this filter expression.</param>
        public void Add(string name, string expression, Bitmap img)
        {
            ExpressionAndImage expImg = new ExpressionAndImage(name);
            expImg.Expression = expression;
            expImg.Image = img;
            expressionFilters.Add(name, expImg);
        }

        internal class ExpressionAndImage
        {
            string expression;
            Bitmap image;
            string name;

            public ExpressionAndImage(string name)
            {
                this.name = name;
            }

            public Bitmap Image
            {
                get { return image; }
                set { image = value; }
            }
            public string Expression
            {
                get { return expression; }
                set { expression = value; }
            }

            public string Name
            {
                get { return name; }
                set { name = value; }
            }

        }

        #region IEnumerator Members
        int index = -1;
        string[] keys = null;
        public object Current
        {
            get
            {
                if (keys == null)
                    return null;

                return this[keys[index]];
            }
        }

        public bool MoveNext()
        {
            if (index == -1)
            {
                if (keys != null)
                    keys = null;
                keys = new string[expressionFilters.Count];
                expressionFilters.Keys.CopyTo(keys, 0);
            }

            if (index < expressionFilters.Count - 1)
            {
                index++;
                return true;
            }
            else
            {
                index = -1;
                return false;
            }
        }

        public void Reset()
        {
            index = -1;
        }

        #endregion

        #region IEnumerable Members

        public IEnumerator GetEnumerator()
        {
            return (IEnumerator)this;
        }

        #endregion
    }
    #endregion

}
